@@ -2,7 +2,7 @@
     adm1025.c - Part of lm_sensors, Linux kernel modules for hardware
                monitoring
     Copyright (c) 2000 Chen-Yuan Wu <gwu@esoft.com>
-    Copyright (c) 2003 Jean Delvare <khali@linux-fr.org>
+    Copyright (c) 2003-2004 Jean Delvare <khali@linux-fr.org>
 
     Based on the adm9240 driver.
 
@@ -26,15 +26,9 @@
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
-#include <linux/ioport.h>
-#include <linux/sysctl.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
-#include <asm/io.h>
 #include "version.h"
 #include "sensors_vid.h"
 
@@ -47,7 +41,7 @@ static unsigned int normal_isa[] = { SENSORS_ISA_END };
 static unsigned int normal_isa_range[] = { SENSORS_ISA_END };
 
 /* Insmod parameters */
-SENSORS_INSMOD_1(adm1025);
+SENSORS_INSMOD_2(adm1025, ne1619);
 
 /* Many ADM1025 constants specified below */
 
@@ -118,19 +112,12 @@ SENSORS_INSMOD_1(adm1025);
 
 #define ALARMS_FROM_REG(val) (val)
 
-/* Initial limits (10% deviation) */
-#define ADM1025_INIT_IN 192
-#define ADM1025_INIT_IN_MIN ((ADM1025_INIT_IN *  9 + 5) / 10)
-#define ADM1025_INIT_IN_MAX ((ADM1025_INIT_IN * 11 + 5) / 10)
-
-#define ADM1025_INIT_TEMP_HIGH 650
-#define ADM1025_INIT_TEMP_LOW 50
-
 /* For each registered ADM1025, we need to keep some data in memory. That
    data is pointed to by adm1025_list[NR]->data. The structure itself is
    dynamically allocated, at the same time when a new adm1025 client is
    allocated. */
 struct adm1025_data {
+	struct i2c_client client;
 	int sysctl_id;
 	enum chips type;
 
@@ -170,9 +157,6 @@ static void adm1025_vid(struct i2c_client *client, int operation,
 static void adm1025_vrm(struct i2c_client *client, int operation,
 			int ctl_name, int *nrels_mag, long *results);
 
-/* I choose here for semi-static ADM1025 allocation. Complete dynamic
-   allocation could also be used; the code needed for this would probably
-   take more memory than the datastructure takes now. */
 static int adm1025_id = 0;
 
 static struct i2c_driver adm1025_driver = {
@@ -276,14 +260,12 @@ static int adm1025_detect(struct i2c_adapter *adapter, int address,
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access adm1025_{read,write}_value. */
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct adm1025_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct adm1025_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct adm1025_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -299,24 +281,34 @@ static int adm1025_detect(struct i2c_adapter *adapter, int address,
 
 	/* Determine the chip type. */
 	if (kind <= 0) {
-		i = i2c_smbus_read_byte_data(new_client, ADM1025_REG_COMPANY_ID);
-		if (((i2c_smbus_read_byte_data(new_client, ADM1025_REG_DIE_REV) & 0xf0) == 0x20)
-		 && ((i == 0x41)   /* ADM1025 */
-		  || (i == 0xA1))) /* NE1619 */
-			kind = adm1025;
-		else {
-			if (kind == 0)
-				printk
-				    ("adm1025.o: Ignoring 'force' parameter for unknown chip at "
-				     "adapter %d, address 0x%02x\n",
-				     i2c_adapter_id(adapter), address);
-			goto ERROR1;
+		u8 man_id, chip_id;
+		
+		man_id = i2c_smbus_read_byte_data(new_client,
+			 ADM1025_REG_COMPANY_ID);
+		chip_id = i2c_smbus_read_byte_data(new_client,
+			  ADM1025_REG_DIE_REV);
+		
+		if (man_id == 0x41) { /* Analog Devices */
+			if ((chip_id & 0xF0) == 0x20) /* ADM1025 */
+				kind = adm1025;
+		} else if (man_id == 0xA1) { /* Philips */
+			if (address != 0x2E
+			 && (chip_id & 0xF0) == 0x20) /* NE1619 */
+				kind = ne1619;
 		}
 	}
 
+	if (kind <= 0) { /* Identification failed */
+		printk("adm1025.o: Unsupported chip.\n");
+		goto ERROR1;
+	}
+
 	if (kind == adm1025) {
 		type_name = "adm1025";
 		client_name = "ADM1025 chip";
+	} else if (kind == ne1619) {
+		type_name = "ne1619";
+		client_name = "NE1619 chip";		
 	} else {
 #ifdef DEBUG
 		printk("adm1025.o: Internal error: unknown kind (%d)?!?",
@@ -357,7 +349,7 @@ static int adm1025_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -375,43 +367,21 @@ static int adm1025_detach_client(struct i2c_client *client)
 		return err;
 	}
 
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
-
 }
 
-/* Called when we have found a new ADM1025. It should set limits, etc. */
+/* Called when we have found a new ADM1025. */
 static void adm1025_init_client(struct i2c_client *client)
 {
 	struct adm1025_data *data = client->data;
-	u8 reg, nr;
+	u8 reg;
 
 	data->vrm = DEFAULT_VRM;
-	/* Restore power-up default values (configuration and status
-	   registers). This makes some other initializations
-	   unnecessary. Set the chip in standby mode until the
-	   limits are properly set. Note that we take great care not to
-	   change the bits 1-6 of the configuration register, since
-	   they hold data we want to preserve. */
+
+	/* Start monitoring */
 	reg = i2c_smbus_read_byte_data(client, ADM1025_REG_CONFIG);
-	i2c_smbus_write_byte_data(client, ADM1025_REG_CONFIG, 0x80);
-
-	for (nr = 0; nr < 6; nr++) {
-		i2c_smbus_write_byte_data(client, ADM1025_REG_IN_MIN(nr),
-				    data->in_min[nr] = IN_TO_REG(ADM1025_INIT_IN_MIN));
-		i2c_smbus_write_byte_data(client, ADM1025_REG_IN_MAX(nr),
-				    data->in_max[nr] = IN_TO_REG(ADM1025_INIT_IN_MAX));
-	}
-
-	for (nr = 0; nr < 2; nr++) {
-		i2c_smbus_write_byte_data(client, ADM1025_REG_TEMP_HIGH(nr),
-			    	data->temp_high[nr]=TEMP_TO_REG(ADM1025_INIT_TEMP_HIGH));
-		i2c_smbus_write_byte_data(client, ADM1025_REG_RTEMP_LOW,
-			    	data->temp_low[nr]=TEMP_TO_REG(ADM1025_INIT_TEMP_LOW));
-	}
-
-	/* Restore configuration and start monitoring */
 	i2c_smbus_write_byte_data(client, ADM1025_REG_CONFIG, (reg|0x01)&0x7F);
 }
 
