@@ -78,12 +78,9 @@ SENSORS_INSMOD_1(ds1621);
 #define ITEMP_FROM_REG(val) ((((val & 0x7fff) >> 8)) | \
                             ((val & 0x8000)?-256:0))
 
-/* Initial values */
-#define DS1621_INIT_TEMP_OVER 600
-#define DS1621_INIT_TEMP_HYST 0 /* 500 would cause an alarm at room temp. */
-
 /* Each client has this additional data */
 struct ds1621_data {
+	struct i2c_client client;
 	int sysctl_id;
 
 	struct semaphore update_lock;
@@ -103,7 +100,6 @@ static int ds1621_detect(struct i2c_adapter *adapter, int address,
 static void ds1621_init_client(struct i2c_client *client);
 static int ds1621_detach_client(struct i2c_client *client);
 
-static u16 swap_bytes(u16 val);
 static int ds1621_read_value(struct i2c_client *client, u8 reg);
 static int ds1621_write_value(struct i2c_client *client, u8 reg, u16 value);
 static void ds1621_temp(struct i2c_client *client, int operation,
@@ -187,20 +183,19 @@ int ds1621_detect(struct i2c_adapter *adapter, int address,
 #endif
 
 	if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_BYTE_DATA |
-				     I2C_FUNC_SMBUS_WORD_DATA))
+				     I2C_FUNC_SMBUS_WORD_DATA |
+				     I2C_FUNC_SMBUS_WRITE_BYTE))
 		    goto ERROR0;
 
 	/* OK. For now, we presume we have a valid client. We now create the
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access ds1621_{read,write}_value. */
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct ds1621_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct ds1621_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct ds1621_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -260,7 +255,7 @@ int ds1621_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -278,17 +273,12 @@ static int ds1621_detach_client(struct i2c_client *client)
 		return err;
 	}
 
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
 
 
-static u16 swap_bytes(u16 val)
-{
-	return (val >> 8) | (val << 8);
-}
-
 /* All registers are word-sized, except for the configuration register.
    DS1621 uses a high-byte first convention, which is exactly opposite to
    the usual practice. */
@@ -298,7 +288,7 @@ static int ds1621_read_value(struct i2c_client *client, u8 reg)
 	    || (reg == DS1621_REG_TEMP_SLOPE))
 		return i2c_smbus_read_byte_data(client, reg);
 	else
-		return swap_bytes(i2c_smbus_read_word_data(client, reg));
+		return swab16(i2c_smbus_read_word_data(client, reg));
 }
 
 /* All registers are word-sized, except for the configuration register.
@@ -313,19 +303,13 @@ static int ds1621_write_value(struct i2c_client *client, u8 reg, u16 value)
 	    || (reg == DS1621_REG_TEMP_SLOPE))
 		return i2c_smbus_write_byte_data(client, reg, value);
 	else
-		return i2c_smbus_write_word_data(client, reg,
-						 swap_bytes(value));
+		return i2c_smbus_write_word_data(client, reg, swab16(value));
 }
 
 static void ds1621_init_client(struct i2c_client *client)
 {
 	int reg;
 
-	/* Initialize the DS1621 chip */
-	ds1621_write_value(client, DS1621_REG_TEMP_OVER,
-			 TEMP_TO_REG(DS1621_INIT_TEMP_OVER));
-	ds1621_write_value(client, DS1621_REG_TEMP_HYST,
-			 TEMP_TO_REG(DS1621_INIT_TEMP_HYST));
 	reg = ds1621_read_value(client, DS1621_REG_CONF);
 	/* start the continous conversion */
 	if(reg & 0x01)
