@@ -47,15 +47,11 @@
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
 #include <linux/ioport.h>
-#include <linux/sysctl.h>
 #include <linux/pci.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
 #include <asm/io.h>
 #include "version.h"
 
@@ -138,13 +134,13 @@ static int blacklist[] = {
 #define SIS5595_REG_ALARM2 0x42
 #define SIS5595_REG_FANDIV 0x47
 
-/* Conversions. Rounding and limit checking is only done on the TO_REG
+/* Conversions. Limit checking is only done on the TO_REG
    variants. Note that you should be a bit careful with which arguments
    these macros are called: arguments may be evaluated more than once.
    Fixing this is just not worth it. */
 
 #define IN_TO_REG(val)  (SENSORS_LIMIT((((val) * 10 + 8)/16),0,255))
-#define IN_FROM_REG(val) (((val) *  16) / 10)
+#define IN_FROM_REG(val) (((val) *  16 + 5) / 10)
 
 static inline u8 FAN_TO_REG(long rpm, int div)
 {
@@ -169,48 +165,12 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define DIV_FROM_REG(val) (1 << (val))
 #define DIV_TO_REG(val) ((val)==8?3:(val)==4?2:(val)==1?0:1)
 
-/* Initial limits. To keep them sane, we use the 'standard' translation as
-   specified in the SIS5595 sheet. Use the config file to set better limits. */
-#define SIS5595_INIT_IN_0 (((1200)  * 10)/38)
-#define SIS5595_INIT_IN_1 (((500)   * 100)/168)
-#define SIS5595_INIT_IN_2 330
-#define SIS5595_INIT_IN_3 250
-#define SIS5595_INIT_IN_4 250
-
-#define SIS5595_INIT_IN_PERCENTAGE 10
-
-#define SIS5595_INIT_IN_MIN_0 \
-        (SIS5595_INIT_IN_0 - SIS5595_INIT_IN_0 * SIS5595_INIT_IN_PERCENTAGE / 100)
-#define SIS5595_INIT_IN_MAX_0 \
-        (SIS5595_INIT_IN_0 + SIS5595_INIT_IN_0 * SIS5595_INIT_IN_PERCENTAGE / 100)
-#define SIS5595_INIT_IN_MIN_1 \
-        (SIS5595_INIT_IN_1 - SIS5595_INIT_IN_1 * SIS5595_INIT_IN_PERCENTAGE / 100)
-#define SIS5595_INIT_IN_MAX_1 \
-        (SIS5595_INIT_IN_1 + SIS5595_INIT_IN_1 * SIS5595_INIT_IN_PERCENTAGE / 100)
-#define SIS5595_INIT_IN_MIN_2 \
-        (SIS5595_INIT_IN_2 - SIS5595_INIT_IN_2 * SIS5595_INIT_IN_PERCENTAGE / 100)
-#define SIS5595_INIT_IN_MAX_2 \
-        (SIS5595_INIT_IN_2 + SIS5595_INIT_IN_2 * SIS5595_INIT_IN_PERCENTAGE / 100)
-#define SIS5595_INIT_IN_MIN_3 \
-        (SIS5595_INIT_IN_3 - SIS5595_INIT_IN_3 * SIS5595_INIT_IN_PERCENTAGE / 100)
-#define SIS5595_INIT_IN_MAX_3 \
-        (SIS5595_INIT_IN_3 + SIS5595_INIT_IN_3 * SIS5595_INIT_IN_PERCENTAGE / 100)
-#define SIS5595_INIT_IN_MIN_4 \
-        (SIS5595_INIT_IN_4 - SIS5595_INIT_IN_4 * SIS5595_INIT_IN_PERCENTAGE / 100)
-#define SIS5595_INIT_IN_MAX_4 \
-        (SIS5595_INIT_IN_4 + SIS5595_INIT_IN_4 * SIS5595_INIT_IN_PERCENTAGE / 100)
-
-#define SIS5595_INIT_FAN_MIN_1 3000
-#define SIS5595_INIT_FAN_MIN_2 3000
-
-#define SIS5595_INIT_TEMP_OVER 600
-#define SIS5595_INIT_TEMP_HYST 100
-
 /* For the SIS5595, we need to keep some data in memory. That
    data is pointed to by sis5595_list[NR]->data. The structure itself is
    dynamically allocated, at the time when the new sis5595 client is
    allocated. */
 struct sis5595_data {
+	struct i2c_client client;
 	struct semaphore lock;
 	int sysctl_id;
 
@@ -427,13 +387,11 @@ int sis5595_detect(struct i2c_adapter *adapter, int address,
 		}
 	}
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct sis5595_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct sis5595_data), GFP_KERNEL))) {
 		return -ENOMEM;
 	}
 
-	data = (struct sis5595_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	init_MUTEX(&data->lock);
 	new_client->data = data;
@@ -486,7 +444,7 @@ int sis5595_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
 	release_region(address, SIS5595_EXTENT);
-	kfree(new_client);
+	kfree(data);
 	return err;
 }
 
@@ -504,7 +462,7 @@ static int sis5595_detach_client(struct i2c_client *client)
 	}
 
 	release_region(client->addr, SIS5595_EXTENT);
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
@@ -533,52 +491,14 @@ static int sis5595_write_value(struct i2c_client *client, u8 reg, u8 value)
 	return 0;
 }
 
-/* Called when we have found a new SIS5595. It should set limits, etc. */
+/* Called when we have found a new SIS5595. */
 static void sis5595_init_client(struct i2c_client *client)
 {
-	struct sis5595_data *data = client->data;
-
-	/* Reset all except Watchdog values and last conversion values
-	   This sets fan-divs to 2, among others */
-	sis5595_write_value(client, SIS5595_REG_CONFIG, 0x80);
-
-	sis5595_write_value(client, SIS5595_REG_IN_MIN(0),
-			    IN_TO_REG(SIS5595_INIT_IN_MIN_0));
-	sis5595_write_value(client, SIS5595_REG_IN_MAX(0),
-			    IN_TO_REG(SIS5595_INIT_IN_MAX_0));
-	sis5595_write_value(client, SIS5595_REG_IN_MIN(1),
-			    IN_TO_REG(SIS5595_INIT_IN_MIN_1));
-	sis5595_write_value(client, SIS5595_REG_IN_MAX(1),
-			    IN_TO_REG(SIS5595_INIT_IN_MAX_1));
-	sis5595_write_value(client, SIS5595_REG_IN_MIN(2),
-			    IN_TO_REG(SIS5595_INIT_IN_MIN_2));
-	sis5595_write_value(client, SIS5595_REG_IN_MAX(2),
-			    IN_TO_REG(SIS5595_INIT_IN_MAX_2));
-	sis5595_write_value(client, SIS5595_REG_IN_MIN(3),
-			    IN_TO_REG(SIS5595_INIT_IN_MIN_3));
-	sis5595_write_value(client, SIS5595_REG_IN_MAX(3),
-			    IN_TO_REG(SIS5595_INIT_IN_MAX_3));
-	sis5595_write_value(client, SIS5595_REG_FAN_MIN(1),
-			    FAN_TO_REG(SIS5595_INIT_FAN_MIN_1, 2));
-	sis5595_write_value(client, SIS5595_REG_FAN_MIN(2),
-			    FAN_TO_REG(SIS5595_INIT_FAN_MIN_2, 2));
-	if(data->maxins == 4) {
-		sis5595_write_value(client, SIS5595_REG_IN_MIN(4),
-				    IN_TO_REG(SIS5595_INIT_IN_MIN_4));
-		sis5595_write_value(client, SIS5595_REG_IN_MAX(4),
-				    IN_TO_REG(SIS5595_INIT_IN_MAX_4));
-	} else {
-		sis5595_write_value(client, SIS5595_REG_TEMP_OVER,
-				    TEMP_TO_REG(SIS5595_INIT_TEMP_OVER));
-		sis5595_write_value(client, SIS5595_REG_TEMP_HYST,
-				    TEMP_TO_REG(SIS5595_INIT_TEMP_HYST));
-	}
+	u8 reg;
 
 	/* Start monitoring */
-	sis5595_write_value(client, SIS5595_REG_CONFIG,
-			    (sis5595_read_value(client, SIS5595_REG_CONFIG)
-			     & 0xf7) | 0x01);
-
+	reg = i2c_smbus_read_byte_data(client, SIS5595_REG_CONFIG);
+	sis5595_write_value(client, SIS5595_REG_CONFIG, (reg|0x01)&0x7F);
 }
 
 static void sis5595_update_client(struct i2c_client *client)
